--[[
    hyper-ecs, an ECS framework for LÖVE
    Copyright (C) 2018 Evan Hemsley

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--]]

local class = require "middleclass"

local Engine = class('Engine')

function Engine:initialize(component_types)
    self.component_types = component_types
    self.tracked_entities = {}
end

function Engine:check_and_register_entity(entity)
    if self.tracked_entities[entity] ~= nil then return end

    local has_all_components = true

    for _, component_type in ipairs(self.component_types) do
        has_all_components = has_all_components and entity:has_subcomponent_of(component_type)
    end

    if has_all_components then
        self.tracked_entities[entity] = entity
    end
end

return Engine