--[[
    hyper-ecs, an ECS framework for LÖVE
    Copyright (C) 2018 Evan Hemsley

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--]]

local class = require "middleclass"

local Entity = class('Entity')

function Entity:initialize(id, world)
    self.id = id
    self.world = world

    self.component_map = {}
    self.component_types = {}
end

function Entity:add_component(type)
    local component = self.world:create_component(type, self)
    self.component_map[type] = component
    self.component_types[type] = type
    self.world:check_and_register_entity(self)
    return component
end

function Entity:get_component(type)
    return self.component_map[type]
end

function Entity:has_component(type)
    return self.component_types[type] ~= nil
end

-- subcomponent is defined as equal or lower to given type
function Entity:has_subcomponent_of(type)
    if self:has_component(type) then return true end

    for _, v in pairs(self.component_types) do
        if v:isSubclassOf(type) then return true end
    end

    return false
end

return Entity