local class = require "middleclass"

local World = require('hyper').World
local Component = require('hyper').Component

local helper = require('spec.helper')
assert:register(
    "assertion",
    "has_value",
    helper.has_value,
    "assertion.has_value.positive",
    "assertion.has_value.negative"
)

describe('Engine', function()
    local A = class('A', Component)
    local B = class('B', Component)
    local C = class('C', Component)

    local SubA = A:subclass('SubA')

    describe('check_and_register_entity', function()
        local world = World:new()
        local engine = world:create_engine({A, B})

        it('tracks the entity if contains all components', function()
            local entity_to_track = world:create_entity()
            entity_to_track:add_component(A)
            entity_to_track:add_component(B)
            assert.has_value(engine.tracked_entities, entity_to_track)
        end)

        it ('tracks the entity if it contains subcomponents', function()
            local entity_to_track = world:create_entity()
            entity_to_track:add_component(SubA)
            entity_to_track:add_component(B)
            assert.has_value(engine.tracked_entities, entity_to_track)
        end)

        it('does not track the entity if it does not have all components', function()
            local entity_not_to_track = world:create_entity()
            entity_not_to_track:add_component(A)
            entity_not_to_track:add_component(C)
            assert.is_not.has_value(engine.tracked_entities, entity_not_to_track)
        end)
    end)
end)