local class = require "middleclass"
local World = require('hyper').World
local Component = require('hyper').Component

describe('Entity', function()
    local MockComponent = class('MockComponent', Component)

    describe('add_component', function()
        local world = World:new()
        local entity = world:create_entity()
        local component = entity:add_component(MockComponent)

        it('adds the component type to the map', function()
            assert.are.same(entity.component_map[MockComponent], component)
        end)

        it('adds the component type to the component type set', function()
            assert.are.same(entity.component_types[MockComponent], MockComponent)
        end)
    end)

    describe('get_component', function()
        it('returns a component table by type', function()
            local world = World:new()
            local entity = world:create_entity()
            local component = entity:add_component(MockComponent)

            assert.are.same(entity:get_component(MockComponent), component)
        end)
    end)

    describe('has_component', function()
        describe('can check if it has a component of given type', function()
            it('returns true if it does', function()
                local world = World:new()
                local entity = world:create_entity()
                entity:add_component(MockComponent)
                assert.is_true(entity:has_component(MockComponent))
            end)

            it('returns false if it doesnt', function()
                local world = World:new()
                local entity = world:create_entity()
                assert.is_false(entity:has_component(MockComponent))
            end)
        end)
    end)

    describe('has_subcomponent_of', function()
        local world = World:new()

        local Fruit = Component:subclass('Fruit')
        local Banana = Fruit:subclass('Banana')

        it('returns true if it does', function()
            local entity = world:create_entity()

            entity:add_component(Banana)
            assert.is_true(entity:has_subcomponent_of(Fruit))
        end)

        it('returns false if it doesnt', function()
            local Unrelated = Component:subclass('Unrelated')
            local entity = world:create_entity()

            entity:add_component(Unrelated)
            assert.is_false(entity:has_subcomponent_of(Fruit))
        end)
    end)
end)